/**
 * Javascript code for finding all tables with classname "striped" and
 * dynamically striping their row colors, and finding all tables with
 * classname "sortable" and making them dynamically sortable.
 *
 * $Horde: whups/js/tables.js,v 1.1 2005/07/08 20:00:43 chuck Exp $
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 */

/* We do everything onload so that the entire document is present
 * before we start searching it for tables. */
if (window.addEventListener) {
    window.addEventListener('load', table_init, false);
} else if (window.attachEvent) {
    window.attachEvent('onload', table_init);
} else if (window.onload != null) {
    var old_onload = window.onload;
    window.onload = function(e)
    {
        old_onload(e);
        table_init();
    };
} else {
    window.onload = table_init;
}

var SORT_COLUMN_INDEX;

function table_init()
{
    if (!document.getElementsByTagName) {
        return;
    }
    tables = document.getElementsByTagName('table');
    for (i = 0; i < tables.length; i++) {
        if (tables[i].className.indexOf('striped') != -1) {
            table_stripe(tables[i]);
        }
        if (tables[i].className.indexOf('sortable') != -1 && tables[i].id) {
            table_makeSortable(tables[i]);
        }
    }
}

function table_stripe(table)
{
    // The flag we'll use to keep track of whether the current row is
    // odd or even.
    var even = false;

    // Tables can have more than one tbody element; get all child
    // tbody tags and interate through them.
    var tbodies = table.childNodes;
    for (var c = 0; c < tbodies.length; c++) {
        if (tbodies[c].tagName == 'TBODY') {
            var trs = tbodies[c].childNodes;
            for (var i = 0; i < trs.length; i++) {
                if (trs[i].tagName == 'TR') {
                    if (trs[i].className) {
                        trs[i].className += ' ';
                    }
                    trs[i].className += even ? 'rowEven' : 'rowOdd';

                    // Flip from odd to even, or vice-versa.
                    even = !even;
                }
            }
        }
    }
}

function table_makeSortable(table)
{
    if (table.rows && table.rows.length > 0) {
        var firstRow = table.rows[0];
    }
    if (!firstRow) {
        return;
    }

    // We have a first row: assume its the header, and make its
    // contents clickable links.
    for (var i = 0; i < firstRow.cells.length; i++) {
        var cell = firstRow.cells[i];
        cell.onclick = function(e)
        {
            var e = e || window.event;
            if (e.target) {
                if (e.target.nodeType == 3) {
                    e.target = e.target.parentNode;
                }
            } else if (e.srcElement) {
                e.target = e.srcElement;
            }
            table_resortTable(e.target);
            return false;
        }
        cell.innerHTML = cell.innerHTML + '<span class="sortarrow">&nbsp;</span>';
    }
}

function table_getInnerText(el)
{
    if (typeof el == "string") {
        return el;
    }
    if (typeof el == "undefined") {
        return el;
    }
    if (el.innerText) {
        // Not needed but it is faster.
        return el.innerText;
    }

    var str = "";
    var cs = el.childNodes;
    var l = cs.length;
    for (var i = 0; i < l; i++) {
        switch (cs[i].nodeType) {
        case 1:
            // ELEMENT_NODE
            str += table_getInnerText(cs[i]);
            break;

        case 3:
            // TEXT_NODE
            str += cs[i].nodeValue;
            break;
        }
    }

    return str;
}

function table_resortTable(th)
{
    th = getParent(th, 'TH');
    // Get the span.
    var span;
    for (var ci = 0; ci < th.childNodes.length; ci++) {
        if (th.childNodes[ci].tagName && th.childNodes[ci].tagName.toLowerCase() == 'span')
            span = th.childNodes[ci];
    }
    var column = th.cellIndex;
    var table = getParent(th, 'TABLE');

    // Work out a type for the column
    if (table.rows.length <= 1) return;
    var itm = table_getInnerText(table.rows[1].cells[column]);
    sortfn = table_sort_caseinsensitive;
    if (itm.match(/^\d\d[\/-]\d\d[\/-]\d\d\d\d$/)) sortfn = table_sort_date;
    if (itm.match(/^\d\d[\/-]\d\d[\/-]\d\d$/)) sortfn = table_sort_date;
    if (itm.match(/^[$]/)) sortfn = table_sort_currency;
    if (itm.match(/^[\d\.]+$/)) sortfn = table_sort_numeric;
    SORT_COLUMN_INDEX = column;
    var firstRow = new Array();
    var newRows = new Array();
    for (i = 0; i < table.rows[0].length; i++) {
        firstRow[i] = table.rows[0][i];
    }
    for (j =1; j < table.rows.length; j++) {
        newRows[j - 1] = table.rows[j];
    }

    newRows.sort(sortfn);

    if (span.getAttribute("sortdir") == 'down') {
        ARROW = '&uarr;';
        newRows.reverse();
        span.setAttribute('sortdir', 'up');
    } else {
        ARROW = '&darr;';
        span.setAttribute('sortdir', 'down');
    }

    // We appendChild rows that already exist to the tbody, so it
    // moves them rather than creating new ones. Don't do sortbottom
    // rows.
    for (i = 0; i < newRows.length; i++) {
        if (!newRows[i].className || (newRows[i].className && (newRows[i].className.indexOf('sortbottom') == -1))) {
            table.tBodies[0].appendChild(newRows[i]);
        }
    }
    // Do sortbottom rows only.
    for (i = 0; i < newRows.length; i++) {
        if (newRows[i].className && (newRows[i].className.indexOf('sortbottom') != -1)) table.tBodies[0].appendChild(newRows[i]);
    }

    // Delete any other arrows there may be showing
    var allspans = document.getElementsByTagName("span");
    for (var ci = 0; ci < allspans.length; ci++) {
        if (allspans[ci].className == 'sortarrow') {
            if (getParent(allspans[ci], "table") == getParent(th, "table")) {
                // In the same table as us?
                allspans[ci].innerHTML = '&nbsp;';
            }
        }
    }

    span.innerHTML = ARROW;
}

function getParent(el, pTagName)
{
    if (el == null) {
        return null;
    } else if (el.nodeType == 1 && el.tagName.toLowerCase() == pTagName.toLowerCase()) {
        // Gecko bug, supposed to be uppercase.
        return el;
    } else {
        return getParent(el.parentNode, pTagName);
    }
}

function table_sort_date(a, b)
{
    // Two digit years less than 50 are treated as 20XX, greater than
    // 50 are treated as 19XX.
    aa = table_getInnerText(a.cells[SORT_COLUMN_INDEX]);
    bb = table_getInnerText(b.cells[SORT_COLUMN_INDEX]);
    if (aa.length == 10) {
        dt1 = aa.substr(6, 4) + aa.substr(3, 2) + aa.substr(0, 2);
    } else {
        yr = aa.substr(6, 2);
        if (parseInt(yr) < 50) {
            yr = '20' + yr;
        } else {
            yr = '19' + yr;
        }
        dt1 = yr+aa.substr(3, 2) + aa.substr(0, 2);
    }
    if (bb.length == 10) {
        dt2 = bb.substr(6, 4) + bb.substr(3, 2) + bb.substr(0, 2);
    } else {
        yr = bb.substr(6, 2);
        if (parseInt(yr) < 50) {
            yr = '20' + yr;
        } else {
            yr = '19' + yr;
        }
        dt2 = yr + bb.substr(3, 2) + bb.substr(0, 2);
    }
    if (dt1 == dt2) {
        return 0;
    } else if (dt1 < dt2) {
        return -1;
    }
    return 1;
}

function table_sort_currency(a, b)
{
    aa = table_getInnerText(a.cells[SORT_COLUMN_INDEX]).replace(/[^0-9.]/g, '');
    bb = table_getInnerText(b.cells[SORT_COLUMN_INDEX]).replace(/[^0-9.]/g, '');
    return parseFloat(aa) - parseFloat(bb);
}

function table_sort_numeric(a, b)
{
    aa = parseFloat(table_getInnerText(a.cells[SORT_COLUMN_INDEX]));
    if (isNaN(aa)) {
        aa = 0;
    }
    bb = parseFloat(table_getInnerText(b.cells[SORT_COLUMN_INDEX]));
    if (isNaN(bb)) {
        bb = 0;
    }
    return aa - bb;
}

function table_sort_caseinsensitive(a, b)
{
    aa = table_getInnerText(a.cells[SORT_COLUMN_INDEX]).toLowerCase();
    bb = table_getInnerText(b.cells[SORT_COLUMN_INDEX]).toLowerCase();
    if (aa == bb) {
        return 0;
    } else if (aa < bb) {
        return -1;
    }
    return 1;
}

function table_sort_default(a,b)
{
    aa = table_getInnerText(a.cells[SORT_COLUMN_INDEX]);
    bb = table_getInnerText(b.cells[SORT_COLUMN_INDEX]);
    if (aa == bb) {
        return 0;
    } else if (aa < bb) {
        return -1;
    }
    return 1;
}
